#pragma once

#include "config.h"
#include "WatchyDisplay.h"
#include "WatchFeatures/WatchFeatures.h"
#include <WiFi.h>
#include <WiFiUdp.h>
#include <NTPClient.h>
#include "bma.h"

class Watchy
{
public:
  Watchy();
  void Wake();
  void DeepSleep();
  void ConnectWiFi();
  void SyncNTPTime();
  void DisconnectWiFi();
  void ShowWatchFace(bool partialRefresh = false);
  
  void ClearBusyCallback();
  void SetBusyCallback();

  virtual void InitBoot() {}; // Called on first boot
  virtual void InitWake() {}; // Called every time the watch wakes from sleep
  virtual void HandleButtonPress(uint64_t buttonMask) = 0;
  virtual void HandleDoubleTap() {}
  virtual void HandleTilt() {}
  virtual void DrawWatchFace(bool partialRefresh = false) = 0;


protected:
  void InitBootInternal();
  void InitWakeInternal();

  static void DisplayBusyCallback(const void *);

  static WatchyDisplayBase m_displayBase;
  static WatchyDisplay m_display;
  static RTC_DATA_ATTR BMA423 m_sensor;
  WatchFeatures::WatchFeatures m_features;

private:
  void BmaConfig();
  static uint16_t ReadRegister(uint8_t address, uint8_t reg, uint8_t *data, uint16_t len);
  static uint16_t WriteRegister(uint8_t address, uint8_t reg, uint8_t *data, uint16_t len);
};